@@ -10,7 +10,6 @@ import string
 from ConfigParser import ConfigParser
 from pkg_resources import parse_requirements
 from setuptools.package_index import PackageIndex
-from setuptools.archive_util import unpack_archive
 
 from pypi2pkgsys.utils import *
 
@@ -18,7 +17,13 @@ patchdir = os.path.join(os.path.dirname(__file__), 'patches')
 config = ConfigParser()
 config.read(os.path.join(patchdir, 'index.ini'))
 
-popen_fmt = '(cd %s; python setup.py dump | tail -n 1 -)'
+accepted = {}
+if config.has_option('pypi', 'accepted'):
+    accepted = config.get('pypi', 'accepted').split()
+
+# If setup.py write something else by itself, the following code might not
+# work. FIX ME.
+popen_fmt = '(cd %s; python setup.py dump)'
 
 class PYPI2Package(object):
     def __init__(self, PackageSystem, argv):
@@ -61,46 +66,51 @@ class PYPI2Package(object):
             for idx in xrange(len(packages)):
                 pkg = packages[idx]
 
+                if pkg.split('>=')[0].strip() not in accepted:
+                    print '%s is not accepted.' % pkg
+                    continue
+
                 print 'Downloading %s ...' % pkg
                 dist = map(lambda reqobj:
                                pkgidx.fetch_distribution(reqobj, dldir,
                                                          source = True),
                            parse_requirements([pkg]))[0]
                 if dist is None:
-                    print '%s is None!'
+                    print 'Download %s failed!' % pkg
                     continue
 
-                print 'Processing ...', pkg
+                print 'Unpacking ...', pkg
+                cfgmap = smart_archive(dist, unpackdir)
+                unpackpath = cfgmap['unpackpath']
+
                 # Prepare parameters.
-                pkgdir = '%s-%s' % (dist.project_name, dist.version)
-                cfgmap = {}
-                if config.has_section(pkgdir):
-                    for name, value in config.items(pkgdir):
+                print 'Processing ...', pkg
+                pkgnamever = '%s-%s' % (dist.project_name, dist.version)
+                if config.has_section(pkgnamever):
+                    for name, value in config.items(pkgnamever):
                         cfgmap[name] = value
                 if config.has_section(dist.project_name):
                     for name, value in config.items(dist.project_name):
                         if name not in cfgmap: cfgmap[name] = value
                 if not 'patches' in cfgmap: cfgmap['patches'] = []
                 else: cfgmap['patches'] = cfgmap['patches'].split()
-                if 'pkgdir' in cfgmap: pkgdir = cfgmap['pkgdir']
-                else: cfgmap['pkgdir'] = pkgdir
-                cfgmap['pkgpath'] = dist.location
-                cfgmap['pkgfile'] = os.path.basename(dist.location)
-                unpackpath = os.path.join(unpackdir, pkgdir)
-                cfgmap['unpackpath'] = unpackpath
 
-                # Get setup arguments of the dist.
-                if 'mkdir' in cfgmap: unpack_archive(dist.location, unpackpath)
-                else: unpack_archive(dist.location, unpackdir)
-                self._setup_convert(os.path.join(unpackpath, 'setup.py'))
+                # Apply patches.
                 for p in cfgmap['patches']:
                     print 'Applying %s' % p
                     os.system('(cd %s; patch -p0 < %s)' % \
                                   (unpackpath, os.path.join(patchdir, p)))
+                self._setup_convert(os.path.join(unpackpath, 'setup.py'))
+
                 print 'Get distribution args from %s' % unpackpath
                 p = popen2.popen2(popen_fmt % unpackpath)
                 p[1].close()
-                c = p[0].read()
+                ln = p[0].readline()
+                while ln:
+                    if ln.strip() == '**** PyPI2PkgSys ****': break
+                    ln = p[0].readline()
+                if not ln: raise RuntimeError, 'dump does not work.'
+                c = p[0].readline()
                 p[0].close()
                 args = eval(c)
                 shutil.rmtree(unpackpath)
@@ -129,11 +139,9 @@ class PYPI2Package(object):
         ln = setup_fp.readline()
         while ln:
             lnlist = ln.split()
-            if len(lnlist) >= 4 and \
-                    lnlist[0] == 'from' and \
-                    lnlist[1] == 'distutils.core' and \
-                    lnlist[2] == 'import' and \
-                    'setup' in lnlist:
+            if len(lnlist) >= 4 and lnlist[0] == 'from' and \
+                    lnlist[1] == 'distutils.core' and lnlist[2] == 'import' \
+                    and 'setup' in ln:
                 idx = ln.find('distutils.core')
                 ln = ln[:idx] + 'setuptools' + ln[idx + len('distutils.core'):]
                 modified = True
